import React from 'react';
import { render, unmountComponentAtNode } from 'react-dom';

import MediaInsert from '../components/media-insert';
import MediaUpdate from '../components/media-update';
import { random } from '../lib/utility';
import { remove } from '../lib/dom';
import { setDropArea } from '../lib/utility';
import { MediaItem } from '../types/media';

const insertModal = (ctx, { files, tab }) => {
  const id = `media-${random()}`;
  $('body').append(`<div id="${id}"></div>`);
  const mediaTarget = document.querySelector(`#${id}`);


  // Deprecated
  if (ACMS.Config.LiteEditorFeature === true) {
    const editor = $(ctx).data('lite-editor');
    if (editor) {
      if (!editor._isFocused()) {
        editor._getElementByQuery('[data-selector="lite-editor"]').focus();
      }
      editor.saveSelection();
    }
  }

  render(<MediaInsert
    tab={tab}
    files={files}
    onInsert={(items) => {
      if (!items || !items.length) {
        alert('メディアが選択されていません。');
        return;
      }
      const editor = $(ctx).data('lite-editor');
      if ($('.js-media_preview', ctx).length) {
        const id = $('.js-media_id', ctx).data('id');
        if (items.length === 1) {
          $('.js-media-detail', ctx).show();
          $('.js-media_preview-several', ctx).html('');
          const [item] = items;
          $('.js-media-caption', ctx).html(item.media_caption);
          $('.js-media-link', ctx).html(item.media_link);
          $('.js-media-alt', ctx).html(item.media_alt);
          $('.js-media-text', ctx).html(item.media_text);
          $('.js-media_edit-alt', ctx).data('mid', item.media_id).show();
          $('.js-media_id', ctx).html(`<span class="acms-admin-label acms-admin-label-default js-media-label">${item.media_id}</span>`);
          const sizes = item.media_size.split(' x ');
          const landscape = sizes[0] > sizes[1] ? 'true' : 'false';
          $('.js-media_preview', ctx).html(`
          <div class="acms-admin-media-img-container">
            <div class="acms-admin-media-img-wrap${landscape === 'true' ? ' acms-admin-media-img-landscape' : ''}${item.media_type === 'file' ? ' entryFormFileentryFormFile': ''}">
              <img src="${item.media_thumbnail}" class="js-media_preview-img" data-mid="${item.media_id}" />
              <input type="hidden" name="media_id_${id}[]" value="${item.media_id}">
              <div class="acms-admin-media-btn-group">
                <button class="acms-admin-media-edit-btn js-media_edit" data-mid="${item.media_id}">編集</button>
              </div>
            </div>
          </div>
          `);
        } else {
          $('.js-media_edit-alt', ctx).hide();
          $('.js-media-detail', ctx).hide();
          $('.js-media_preview', ctx).html('');
          $('.js-media_id', ctx).html(items.map((item) => `<span class="acms-admin-label acms-admin-label-default acms-admin-margin-right-small">${item.media_id}</span>`));
          $('.js-media_preview-several', ctx).html(`
            ${items.map((item) => {
              const size = item.media_size;
              const sizes = item.media_size.split(' x ');
              const landscape = sizes[0] > sizes[1] ? 'true' : 'false';
              return (`
                <div class="acms-admin-media-unit-item">
                  <div class="acms-admin-media-img-container">
                    <div class="acms-admin-media-img-wrap${landscape === 'true' ? ' acms-admin-media-img-landscape' : ''}${item.media_type === 'file' ? ' entryFormFileentryFormFile': ''}">
                      <img src="${item.media_thumbnail}" class="js-media_preview-img" data-mid="${item.media_id}" />
                      <div class="acms-admin-media-btn-group">
                        <button class="acms-admin-media-edit-btn js-media_edit" data-mid="${item.media_id}">編集</button>
                      </div>
                      <input type="hidden" name="media_id_${id}[]" value="${item.media_id}">
                    </div>
                  </div>
                </div>
              `);
            }).join('')}
          `);
        }
      // deprecated
      } else {
        const $area = $('textarea[name^="text_text_"]', ctx);
        const [ item ] = items;
        const { media_type, media_thumbnail, media_title, media_link } = item;
        if (ACMS.Config.LiteEditorFeature === true) {
          if (editor) {
            if (!editor._isFocused()) {
              editor._getElementByQuery('[data-selector="lite-editor"]').focus();
            }
            editor.restoreSelection();
            if (editor.data.mode === 'markdown') {
              if (media_type === 'image') {
                editor.insertHtmlAtCursor(`![](${media_thumbnail})`);
              } else if (media_type === 'file') {
                editor.insertHtmlAtCursor(`[${media_title}](${media_link})`);
              }
            } else {
              if (media_type === 'image') {
                editor.insertHtmlAtCursor(`<img src="${media_thumbnail}" alt="" />`);
              } else if (media_type === 'file') {
                editor.insertHtmlAtCursor(`<a href="${media_link}">${media_title}</a>`);
              }
            }
            editor.update();
          }
        } else {
          const tagselect = $(ctx).find("[name^='text_tag_']").val();
          if (tagselect === 'markdown') {
            if (media_type === 'image') {
              $area.selection(`![](${media_thumbnail})`);
            } else if (media_type === 'file') {
              $area.selection(`[${media_title}](${media_link})`);
            }
          } else {
            if (media_type === 'image') {
              $area.selection(`<img src="${media_thumbnail}" alt="" />`);
            } else if (media_type === 'file') {
              $area.selection(`<a href="${media_link}">${media_title}</a>`);
            }
          }
        }
      }
      unmountComponentAtNode(mediaTarget);
      remove(mediaTarget);
    }}
    onClose={() => {
      unmountComponentAtNode(mediaTarget);
      remove(mediaTarget);
    }}
  />, mediaTarget);
}

export default (ctx: HTMLElement) => {
  $(ctx).on('click', '.js-media_insert', (e) => {
    e.preventDefault();
    const tab = $(e.target).data('tab');
    insertModal(ctx, {
      files: [],
      tab
    });
  });
  setDropArea(ctx, '.js-drop-media_area', (files) => {
    insertModal(ctx, {
      files,
      tab: 'upload'
    });
  });
  $(ctx).on('click', '.js-media_edit, .js-media_edit-alt', (e) => {
    e.preventDefault();
    const id = `media-${random()}`;
    const mid = $(e.target).data('mid');
    $('body').append(`<div id="${id}"></div>`);
    const mediaTarget = document.querySelector(`#${id}`);
    render(<MediaUpdate
      mid={`${mid}`}
      onClose={() => {
        unmountComponentAtNode(mediaTarget);
        remove(mediaTarget);
      }}
      onUpdate={(media: MediaItem) => {
        const size = media.media_size;
        const sizes = media.media_size.split(' x ');
        const landscape = sizes[0] > sizes[1] ? 'true' : 'false';
        if (landscape === 'true') {
          $('.js-media_preview-img').parent('.acms-admin-media-img-wrap').addClass('acms-media-img-landscape');
        } else {
          $('.js-media_preview-img').parent('.acms-admin-media-img-wrap').removeClass('acms-media-img-landscape');
        }
        if (media.media_type) {
          $('.js-media_preview-img').addClass('entryFormFileentryFormFile');
        } else {
          $('.js-media_preview-img').removeClass('entryFormFileentryFormFile');
        }
        $(`.js-media_preview-img[data-mid="${mid}"]`, ctx).attr('src', media.media_edited);
        $(`.js-media_id-input[value="${mid}"]`, ctx).val(media.media_id);
        $(e.target).data('mid', media.media_id);
        if ($('.js-media_edit', ctx).length === 1) {
          $('.js-media-caption', ctx).html(media.media_caption);
          $('.js-media-link', ctx).html(media.media_link);
          $('.js-media-alt', ctx).html(media.media_alt);
          $('.js-media-text', ctx).html(media.media_text);
          $('.js-media_id', ctx).html(`<span class="acms-admin-label acms-admin-label-default js-media-label">${media.media_id}</span>`);
        } else {
          let label = '';
          $('.js-media_id-input[value="${mid}"]').each((i, target) => {
            label += `<span class="acms-admin-label acms-admin-label-default js-media-label">${$(target).val()}</span>`;
          });
          $('.js-media_id', ctx).html(label);
        }
        $('.js-media-label').each(function() {
          if ($(this).html() === mid) {
            $(this).html(media.media_id);
          }
        });
        unmountComponentAtNode(mediaTarget);
        remove(mediaTarget);
      }}
    />, mediaTarget);
  });
};
